﻿using System;
using System.Collections.Generic;

using BeSagerCore;

public class BzVObbImporter
{
	public Booking
	ImportLine( IBookingBuilder builder, string[] items )
	{
		if ( items[2].StartsWith( "Mitgliedsbeitrag" ) )
		{
			return CreateMembershipFeeBooking( builder, items );
		}
		else if ( -1 < items[2].IndexOf( "Spende" ) )
		{
			return CreateDonationBooking( builder, items );
		}
		else if ( items[2].StartsWith( "Zahlung" ) )
		{
			return CreateBillBooking( builder, items );
		}
		else if ( items[2].StartsWith( "Reisekosten" ) )
		{
			return CreateTravelCostsBooking( builder, items );
		}
		else if ( items[2].StartsWith( "Zuwendung" ) )
		{
			return CreateAllocationBooking( builder, items );
		}
		else if ( items[2].StartsWith( "Transfer" ) )
		{
			return CreateTransferBooking( builder, items );
		}
		else
		{
			return CreatePlaceholderBooking( builder, items );
		}
	}

	private MembershipFeeBooking
	CreateMembershipFeeBooking( IBookingBuilder builder, string[] items )
	{
		MembershipFeeBooking booking = new MembershipFeeBooking( builder );

		booking.PreviousNumber = string.IsNullOrEmpty( items[0] );
		booking.BookingDate	   = GetDate( items[1] );
		booking.Year		   = 2013;

		int pos;
		if ( -1 < (pos = items[2].IndexOf( "20" )) )
		{
			int year = 2013;
			int.TryParse( items[2].Substring( pos ), out year );
			booking.Year = year;
		}

		booking.MemberNumber = GetMemberNumber( items[4] );

		decimal amount = GetAmount( items[5], items[6] );

		if ( amount < 0 )
		{
			amount				= - amount;
			booking.Cancelation = true;
		}

		booking.Amount = amount;

		booking.Complete();

		return booking;
	}

	private DonationBooking
	CreateDonationBooking( IBookingBuilder builder, string[] items )
	{
		DonationBooking booking = new DonationBooking( builder );

		booking.PreviousNumber = string.IsNullOrEmpty( items[0] );
		booking.BookingDate	   = GetDate( items[1] );
		booking.DonorNumber	   = GetMemberNumber( items[4] ).ToString();

		decimal amount = GetAmount( items[5], items[6] );

		if ( amount < 0 )
		{
			amount				= - amount;
			booking.Cancelation = true;
		}

		booking.Amount = amount;

		if ( items[2].StartsWith( "zweckg" ) )
		{
			booking.Purpose = "R99999999";
		}

		booking.Complete();

		return booking;
	}

	private BillBooking
	CreateBillBooking( IBookingBuilder builder, string[] items )
	{
		BillBooking booking = new BillBooking( builder );

		booking.PreviousNumber = string.IsNullOrEmpty( items[0] );
		booking.BookingDate	   = GetDate( items[1] );

		BillDirection direction = BillDirection.Sell;
		decimal		  amount	= GetAmount( items[5], items[6] );

		if ( amount < 0 )
		{
			amount = - amount;
			direction = BillDirection.Buy;
		}
		
		booking.Amount		= amount;
		booking.Direction	= direction;
		booking.BookingText	= items[3];

		booking.Complete();

		return booking;
	}

	private TravelCostsBooking
	CreateTravelCostsBooking( IBookingBuilder builder, string[] items )
	{
		TravelCostsBooking booking = new TravelCostsBooking( builder );

		booking.PreviousNumber = string.IsNullOrEmpty( items[0] );
		booking.BookingDate	   = GetDate( items[1] );
		booking.MemberNumber   = GetMemberNumber( items[4] );
		booking.Amount		   = Math.Abs( GetAmount( items[5], items[6] ) );
		booking.BookingText	   = items[3];
		booking.Comment		   = "Details der Reisekostenerstattung nachtragen!";

		booking.Complete();

		return booking;
	}

	private AllocationBooking
	CreateAllocationBooking( IBookingBuilder builder, string[] items )
	{
		AllocationBooking booking = new AllocationBooking( builder );

		booking.PreviousNumber = string.IsNullOrEmpty( items[0] );
		booking.BookingDate	   = GetDate( items[1] );

		TransferDirection direction = TransferDirection.Receive;
		decimal			  amount	= GetAmount( items[5], items[6] );

		if ( amount < 0 )
		{
			amount = - amount;
			direction = TransferDirection.Send;
		}

		booking.Amount		= amount;
		booking.Direction	= direction;
		booking.BookingText	= items[3];

		return booking;
	}

	private TransferBooking
	CreateTransferBooking( IBookingBuilder builder, string[] items )
	{
		TransferBooking booking = new TransferBooking( builder );

		booking.PreviousNumber = string.IsNullOrEmpty( items[0] );
		booking.BookingDate	   = GetDate( items[1] );

		TransferDirection direction = TransferDirection.Receive;
		decimal			  amount	= GetAmount( items[5], items[6] );

		if ( amount < 0 )
		{
			amount = - amount;
			direction = TransferDirection.Send;
		}

		booking.Amount		= amount;
		booking.Direction	= direction;
		booking.BookingText	= items[3];

		return booking;
	}

	private PlaceholderBooking
	CreatePlaceholderBooking( IBookingBuilder builder, string[] items )
	{
		PlaceholderBooking booking = new PlaceholderBooking( builder );

		booking.PreviousNumber = string.IsNullOrEmpty( items[0] );
		booking.BookingDate	   = GetDate( items[1] );

		TransferDirection direction = TransferDirection.Receive;
		decimal			  amount	= GetAmount( items[5], items[6] );

		if ( amount < 0 )
		{
			amount = - amount;
			direction = TransferDirection.Send;
		}

		booking.Amount	  = amount;
		booking.Direction = direction;

		for ( int i = 0; i < items.Length; i++ )
		{
			if ( 0 < i )
				booking.Comment += " | ";

			booking.Comment += items[i];
		}

		return booking;
	}

	private DateTime
	GetDate( string date )
	{
		DateTime val = DateTime.Now;
		DateTime.TryParse( date, out val );

		return val;
	}

	private int
	GetMemberNumber( string member )
	{
		int val;
		int.TryParse( member, out val );

		return val;
	}

	private decimal
	GetAmount( string debit, string credit )
	{
		decimal amount;

		if ( ! decimal.TryParse( debit, out amount ) )
		{
			if ( decimal.TryParse( credit, out amount ) )
				amount = - amount;
		}

		return amount;
	}
}